﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Drawing.Drawing2D
Imports System.Drawing.Text

<Serializable()> _
Public Class clsDrawingPanel

    Inherits Panel

    Public Event eventObjectListChanged(sender As Object, e As NotifyCollectionChangedEventArgs)

    Private g As Graphics
    Private m_MouseOffset As Size
    Private m_MouseDown As Boolean = False
    Private m_MouseDrag As Boolean = False

    Private m_HoveredObject As clsDrawingObject
    Public Property HoveredObject() As clsDrawingObject
        Get
            Return m_HoveredObject
        End Get
        Set(ByVal value As clsDrawingObject)
            m_HoveredObject = value
        End Set
    End Property

    Private m_colObjects As ObservableCollection(Of clsDrawingObject)
    Public Property colObjects() As ObservableCollection(Of clsDrawingObject)
        Get
            If m_colObjects Is Nothing Then
                m_colObjects = New ObservableCollection(Of clsDrawingObject)
            End If
            Return m_colObjects
        End Get
        Set(ByVal value As ObservableCollection(Of clsDrawingObject))
            m_colObjects = value
        End Set
    End Property

    Private m_SelectedObject As clsDrawingObject
    Public Property SelectedObject() As clsDrawingObject
        Get
            Return m_SelectedObject
        End Get
        Set(ByVal value As clsDrawingObject)
            If value Is m_SelectedObject Then Return
            m_SelectedObject = value
        End Set
    End Property

    Private m_DragObjects As New ObservableCollection(Of clsDrawingObject)
    Public ReadOnly Property DragObjects() As ObservableCollection(Of clsDrawingObject)
        Get
            Return m_DragObjects
        End Get
    End Property

    Public Sub New()
        'Try
        'Me.DoubleBuffered = True
        'SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.OptimizedDoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.ResizeRedraw, True)
        'UpdateStyles()
        'Catch ex As Exception

        'End Try

        Me.BackColor = Color.White
        Me.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle

        If m_colObjects Is Nothing Then
            m_colObjects = New ObservableCollection(Of clsDrawingObject)
        End If

        'AddHandler m_colObjects.CollectionChanged, AddressOf ObjectListChanged
        'AddHandler colObjects.CollectionChanged, AddressOf ObjectListChanged
    End Sub

    Public Sub ObjectListChanged(sender As Object, e As NotifyCollectionChangedEventArgs)
        'RaiseEvent eventObjectListChanged(sender, e)
    End Sub

    Private Sub clsDrawingPanel_MouseDown(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If e.Button = Windows.Forms.MouseButtons.Left And Not m_MouseDown Then
            m_MouseDown = True
            GetSelected(e.Location)
            If Not m_SelectedObject Is Nothing Then
                m_MouseDrag = True
                m_SelectedObject.Location = New Point(m_SelectedObject.Location.X + (e.Location.X - m_SelectedObject.Location.X), _
                                                      m_SelectedObject.Location.Y + (e.Location.Y - m_SelectedObject.Location.Y))
                If Not m_SelectedObject Is Nothing Then Me.Invalidate()
            End If
        End If
    End Sub

    Private Sub clsDrawingPanel_MouseMove(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        GetHovered(e.Location)
        If m_MouseDrag Then
            m_SelectedObject.Location = New Point(m_SelectedObject.Location.X + (e.Location.X - m_SelectedObject.Location.X), _
                                                  m_SelectedObject.Location.Y + (e.Location.Y - m_SelectedObject.Location.Y))
        End If
        If Not m_HoveredObject Is Nothing Then Me.Invalidate()
    End Sub

    Private Sub clsDrawingPanel_MouseUp(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        If Not m_SelectedObject Is Nothing And m_MouseDrag Then
            If Not m_HoveredObject Is Nothing Then
                If m_SelectedObject.OldLocation.X <> m_HoveredObject.OldLocation.X Then
                    m_SelectedObject.Connection = New Rectangle(m_SelectedObject.OldLocation, New Size(m_HoveredObject.Location.X, m_HoveredObject.Location.Y))
                End If
            End If
            m_SelectedObject.Location = m_SelectedObject.OldLocation
            m_SelectedObject.isSelected = False
            m_SelectedObject.isHovered = False
            m_SelectedObject = Nothing
        End If

        m_MouseDown = False
        m_MouseDrag = False
        Me.Invalidate()
    End Sub

    Private Sub clsPaintPanel_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        Using g = e.Graphics
            g.SmoothingMode = SmoothingMode.AntiAlias
            g.PixelOffsetMode = PixelOffsetMode.HighQuality
            g.TextRenderingHint = TextRenderingHint.ClearTypeGridFit

            If m_colObjects Is Nothing Then Exit Sub

            For Each cDO As clsDrawingObject In m_colObjects
                'If cDO.Bounce.IntersectsWith(e.ClipRectangle) Then Return
                cDO.Paint(e)

                If Not cDO.Connection.IsEmpty Then
                    'PaintSingleConnection(e, Pens.Blue, cDO.OldLocation, cDO.Location)
                    PaintSingleConnection(e, Pens.Blue, cDO.Connection.Location, New Point(cDO.Connection.Width, cDO.Connection.Height))
                End If
            Next
        End Using
    End Sub

    Private Sub GetHovered(MPos As Point)
        HoveredObject = Nothing
        For Each cDO As clsDrawingObject In m_colObjects
            If cDO.Bounce.Contains(MPos) And HoveredObject Is Nothing And Not HoveredObject Is SelectedObject Then
                cDO.isHovered = True
                HoveredObject = cDO
            Else
                cDO.isHovered = False
            End If
        Next
    End Sub

    Private Sub GetSelected(MPos As Point)
        SelectedObject = Nothing
        For Each cDO As clsDrawingObject In m_colObjects
            cDO.isHovered = False
            If cDO.Bounce.Contains(MPos) And SelectedObject Is Nothing Then
                cDO.isSelected = True
                SelectedObject = cDO
            Else
                cDO.isSelected = False
            End If
        Next
    End Sub


    Public Sub PaintSingleConnection(ByVal e As System.Windows.Forms.PaintEventArgs, Stift As Pen, ByVal StartPunkt As Point, ByVal EndPunkt As Point)
        Try
            Dim Abstand As Integer = 20
            If Math.Abs(EndPunkt.X - StartPunkt.X) < Abstand Then
                Abstand = Math.Abs(EndPunkt.X - StartPunkt.X)
            End If

            Dim Start As Point = New Point(StartPunkt.X + Abstand, StartPunkt.Y)
            Dim Ende As Point = New Point(EndPunkt.X - Abstand, EndPunkt.Y)

            Dim Mitte As Point

            Mitte = New Point(CInt(StartPunkt.X + (EndPunkt.X - StartPunkt.X) / 2), CInt(StartPunkt.Y + (EndPunkt.Y - StartPunkt.Y) / 2))

            Dim Mitte2 As Point

            Mitte2 = New Point(CInt(StartPunkt.X + (EndPunkt.X - StartPunkt.X) / 4), CInt(StartPunkt.Y + (EndPunkt.Y - StartPunkt.Y) / 2))
            e.Graphics.DrawBezier(Stift, _
                                  New Point(StartPunkt.X, StartPunkt.Y), _
                                 New Point(Start.X, Start.Y), _
                                 New Point(Mitte2.X, Start.Y), _
                                 Mitte)

            Mitte2 = New Point(CInt(StartPunkt.X + ((EndPunkt.X - StartPunkt.X) / 4) * 3), CInt(StartPunkt.Y + (EndPunkt.Y - StartPunkt.Y) / 2))
            e.Graphics.DrawBezier(Stift, _
                                  Mitte, _
                                  New Point(Mitte2.X, Ende.Y), _
                                  New Point(Ende.X, Ende.Y), _
                                  New Point(EndPunkt.X, EndPunkt.Y))
        Catch ex As Exception

        End Try
    End Sub

End Class
